%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author            : Jeroen de Ridder
% Copyright         : J. de Ridder, J.deRidder@tudelft.nl
%                     Delft University of Technology
%                     P.O. Box 5031, 2600 GA Delft,
%          
%                     The Netherlands Cancer Institute
%                     Plesmanlaan 121, 1066 CX, Amsterdam,
%                     The Netherlands
% 
% Reference paper   : de Ridder, J, Kool, J, Uren, A,
%                     Wessels, L, Reinders, M.
%                     Detecting statistically significant 
%                     common insertion sites in retroviral
%                     insertional mutagenesis screens.
%                     PLoS Comput Biol, 2006, 2, e166
%
%  function : CIMPL_scenario_builder()
%  Purpose  : Command line generation of scenarios
%
%  Inputs   : Kernel widths
%           : Number of permutations
%           : (Optional) parsing function
%             
%  Outputs  : Scenario files
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CIMPL_plotter(kernel_width,varargin)
%Check on what platform we are running
if ispc
    DIR_SYM = '\';
else
    DIR_SYM = '/';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DATA LOADING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Use the directory name to infer the scenario name
tmppwd = pwd;
I = strfind(tmppwd,DIR_SYM);
scenario_name = tmppwd(I(end)+5:end);
try
    load(['.' DIR_SYM scenario_name '_preproc'])
    load(['.' DIR_SYM scenario_name '_pval'])
    load(['.' DIR_SYM scenario_name '_null'])
    load(['.' DIR_SYM 'chrom_l'])
catch
    error('One or more required scenario files not present')
end
d1 = dir(['.' DIR_SYM 'annotation.mat']);
d2 = dir(['..' DIR_SYM 'annotation.mat']);
if isempty(d1)
    if isempty(d2)
        error('No annotation present... ');
    else
        load(['..' DIR_SYM 'annotation.mat']);
    end
else
    load(['.' DIR_SYM 'annotation.mat']);
end
handles.annotation_struct = annotation_struct;

handles.chrom_l = chrom_l;
handles.cum_chrom_l = cumsum([0; chrom_l]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% READ INPUTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[val,I] = min(abs(kernel_width-null_struct.kernel_widths));
if val~=0
    warning('No exact match for kernelwidth computed, using closest match');
end
handles.kernel_index = I;
handles.cur_kernel_width = null_struct.kernel_widths(I);
handles.no_kernels = length(null_struct.kernel_widths);
handles.kernel_widths = null_struct.kernel_widths;


for i = 1:2:length(varargin)
    switch varargin{i}
        case 'alpha'
            handles.alpha = varargin{i+1};
        case 'chr'
            if varargin{i+1} == 0
                handles.chr_view = 0;
                handles.start_view = 1;
                handles.end_view = handles.cum_chrom_l(end);
            else
                chr = varargin{i+1};
                handles.start_view = handles.cum_chrom_l(chr);
                handles.end_view = handles.cum_chrom_l(chr+1)-1;
                handles.chr_view = chr;
            end
        case 'start'
            handles.start_view = varargin{i+1} + handles.cum_chrom_l(chr);
        case 'end'
            handles.end_view = varargin{i+1} + handles.cum_chrom_l(chr);
        case 'plotdefaults'
            track_settings = [];
            load(['.' DIR_SYM varargin{i+1}]);
            handles.track_settings = track_settings;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DEFAULTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Set the default values
handles.show_threshold      = 20e6; %zoom level for detail display
handles.exon_show_threshold = 5; %zoom level for exon detail display
handles.gene_name_threshold = 25; %Threshold for showing gene_names
handles.single_click_T      = 100;   %Single click distance threshold
handles.track_height        = 0.75; %Height of a track marker

if ~isfield(handles,'alpha')
    handles.alpha            = 0.05; %Default alpha level
end
if ~isfield(handles,'plotdefaults')
    track_settings = [];
    if isempty(dir(['.' DIR_SYM 'plotdefaults']))
        CIMPL_plotdefaults
    end
    load(['.' DIR_SYM 'plotdefaults']); %Default plot settings
    handles.track_settings = track_settings;
end
if ~isfield(handles,'start_view')
    handles.start_view = 0;
    handles.chr_view = 0;
end
if ~isfield(handles,'end_view')
    handles.end_view = handles.cum_chrom_l(end);
    handles.chr_view = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DATA PREPARATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%annotation_struct
handles.top = annotation_struct.top;
handles.bot = annotation_struct.bot;

%data_struct
handles.insertions = data_struct.median_coord;
handles.chr = data_struct.chr;
handles.orientation = data_struct.ori;
handles.contig_depth = data_struct.contig_depth;

%pval_struct
handles.densities = pval_struct.densities;
handles.sample_points = pval_struct.sample_points;

handles.peak_pos = pval_struct.peak_pos;
handles.peak_height = pval_struct.peak_height;

%null_struct
handles.Tlevel = zeros(handles.no_kernels,1);
for kindex = 1:handles.no_kernels
    no_tests = length(handles.peak_pos{kindex});
    Iy  = find(null_struct.null.yCDF{kindex}>1-(handles.alpha/no_tests));
    if isempty(Iy)
        error('do more permutations');
    elseif Iy(1)==1
        error('do more permutations');
    else
        handles.Tlevel(kindex) = null_struct.null.xCDF{kindex}(Iy(1)+1);
    end
end

[handles.track_struct, handles.dens_struct, handles.scale_struct] = track_processing(handles);

clear data_struct dens_struct null_struct annotation_struct
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIGURE PREPARATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
handles.fighandle = figure(1); clf
set(handles.fighandle,'position',[5 231 1271 701]);

%Make labels
% handles.label_dens = uicontrol('style','text','units',...
%    'normalized','position',[.02 .7 .1 .03]);
tr_offset = 0.473;
tr_step = 0.030;

%Make axes
handles.label_axes = axes('position',[0.01 0.09 0.19 .8],'visible','off');
handles.scale_space_axes = axes('position',[0.2 0.09 0.7 0.25]);
handles.density_axes = axes('position',[0.2 0.5 0.7 0.4]);
handles.track_axes = axes('position',[0.2 0.345 0.7 0.15]);

handles.label_track = zeros(5,1);

handles.label_dens = text(0.9, 0.7,'Density','parent',handles.label_axes,...
    'fontsize',14,'horizontalalignment','right');
handles.label_track(1) = text(0.9, tr_offset-0,'','parent',handles.label_axes,...
    'fontsize',14,'horizontalalignment','right');
handles.label_track(2) = text(0.9, tr_offset-1*tr_step,'','parent',handles.label_axes,...
    'fontsize',14,'horizontalalignment','right');
handles.label_track(3) = text(0.9, tr_offset-2*tr_step,'','parent',handles.label_axes,...
    'fontsize',14,'horizontalalignment','right');
handles.label_track(4) = text(0.9, tr_offset-3*tr_step,'','parent',handles.label_axes,...
    'fontsize',14,'horizontalalignment','right');
handles.label_track(5) = text(0.9, tr_offset-4*tr_step,'','parent',handles.label_axes,...
    'fontsize',14,'horizontalalignment','right');
handles.label_scale_space = text(0.9, 0.2,'','parent',handles.label_axes,...
    'fontsize',14,'horizontalalignment','right');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DO THE PLOTTING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%DENSITY
plot_density(handles);
xlim([handles.start_view handles.end_view]);
scale = handles.cur_kernel_width;
x_start_pos = (handles.end_view-handles.start_view)*0.05+handles.start_view;
y_start_pos = get(handles.density_axes,'ylim');
y_start_pos = y_start_pos(2)*0.9;
text(x_start_pos,y_start_pos,['Scale: ' num2str(scale)]);
line([x_start_pos x_start_pos+scale],...
    [y_start_pos-y_start_pos*0.05 y_start_pos-y_start_pos*0.05],'color','k');
line([x_start_pos x_start_pos],...
    [y_start_pos-y_start_pos*0.07 y_start_pos-y_start_pos*0.03],...
    'color','k');
line([x_start_pos+scale x_start_pos+scale],...
    [y_start_pos-y_start_pos*0.07 y_start_pos-y_start_pos*0.03],...
    'color','k');
set(handles.density_axes,'xtick',[]);

%SCALE SPACE
plot_scale_space(handles);
xlim([handles.start_view handles.end_view]);
%Set the correct xtick
if handles.chr_view~=0
    xlabel(['Chromosome: ' num2str(handles.chr_view)]);
    xt = get(handles.scale_space_axes,'xtick');
    xt = (xt-handles.cum_chrom_l(handles.chr_view))/1000000;
    xt = cellstr(num2str(xt(:)));
    xt = strrep(xt,'.','M');
    set(handles.scale_space_axes,'xticklabel',xt)
else
    set(handles.scale_space_axes,'xtick',[]);
end

%TRACKS
plot_tracks(handles);
xlim([handles.start_view handles.end_view]);
%Remove the other axes ticks from the figure
set(handles.track_axes,'xtick',[],'ytick',[]);
