%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author            : Jeroen de Ridder
% Copyright         : J. de Ridder, J.deRidder@tudelft.nl
%                     Delft University of Technology
%                     P.O. Box 5031, 2600 GA Delft,
%          
%                     The Netherlands Cancer Institute
%                     Plesmanlaan 121, 1066 CX, Amsterdam,
%                     The Netherlands
% 
% Reference paper   : de Ridder, J, Kool, J, Uren, A,
%                     Wessels, L, Reinders, M.
%                     Detecting statistically significant 
%                     common insertion sites in retroviral
%                     insertional mutagenesis screens.
%                     PLoS Comput Biol, 2006, 2, e166
%
%  function : CIMPL_scenario_builder()
%  Purpose  : Command line generation of scenarios
%
%  Inputs   : Kernel widths
%           : Number of permutations
%           : (Optional) parsing function
%             
%  Outputs  : Scenario files
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CIMPL_scenario_builder(kernel_widths,iterations,varargin)

%Check if we need to use the defaults
if isempty(kernel_widths)
    kernel_widths = [50 100 250 500 1000 2500 5000 10000 30000 50000 100000 150000];
end
if isempty(iterations)
    iterations = 2000;
end

%Below some magic numbers
divisor = 10;
sample_window = 2;
sample_period = 0.1;

%Check on what platform we are running
if ispc
    DIR_SYM = '\';
else
    DIR_SYM = '/';
end

%Use the directory name to infer the scenario name and check if this
%scenario has been run before (if there are output files present)
tmppwd = pwd;
I = strfind(tmppwd,DIR_SYM);
scenario_name = tmppwd(I(end)+5:end);

tmpdir = dir;
old_flag = 0;
cimpl_flag = 1;
chrom_flag = 1;
%Loop over all files in the directory
for i = 1:length(tmpdir)
    %Check if there are leftovers from previous runs
    if strcmp([scenario_name '.sif'],tmpdir(i).name) || ...
        strcmp([scenario_name '_null.mat'],tmpdir(i).name) || ...
        strcmp([scenario_name '_preproc.mat'],tmpdir(i).name) || ...
        strcmp([scenario_name '_pval.mat'],tmpdir(i).name)
    old_flag = 1;
    end
    %Check if there is a cimpl file present
    I = strfind(tmpdir(i).name,'.cimpl');
    if ~isempty(I)
        cimpl_flag = 0;
        cimpl_file_name = tmpdir(i).name;
    end
    %Check if there is a chrom_l file present
    I = strfind(tmpdir(i).name,'chrom_l.mat');
    if ~isempty(I)
        chrom_flag = 0;
    end
end


if old_flag == 1
    error('This scenario may already exist, please backup old previous scenario files');
end
if cimpl_flag == 2
    error('No cimpl file was encountered, please supply one..')
end
if chrom_flag == 2
    error('Did not find the chrom_l.mat file... Please supply one in the current directory');
end

load chrom_l
cum_chrom_l = [0; cumsum(chrom_l)];

if isempty(varargin)
   data_struct = cimpl_parse(cimpl_file_name,cum_chrom_l);
else
   data_struct = varargin{1}(cum_chrom_l);
end

%Save the processed data
save([scenario_name '_preproc'], 'data_struct');

insertions      = data_struct.median_coord;
genomelength    = sum(chrom_l);
null = nullperm_fast(insertions,genomelength,...
    kernel_widths,iterations,100,divisor,sample_period,sample_window);

%Save to structure for saving
null_struct.null = null;
null_struct.iterations = iterations;
null_struct.divisor = divisor;
null_struct.kernel_widths = kernel_widths;
null_struct.genomelength = genomelength;
null_struct.sample_period = sample_period;
null_struct.sample_window = sample_window;

save([scenario_name '_null'],'null_struct');

%Compute the p-values
[pvals densities sample_points...
    peak_pos, peak_height] = CIS_analyze(insertions,...
    kernel_widths,null_struct.null,sample_period,sample_window);

%Put everything in a struct
pval_struct.pvals           = pvals;
pval_struct.densities       = densities;
pval_struct.sample_points   = sample_points;
pval_struct.peak_pos        = peak_pos;
pval_struct.peak_height     = peak_height;

save([scenario_name '_pval'],'pval_struct');

%Generate and Save the Scenario Info File
sif{1,1} = 'Date';
sif{2,1} = 'Time';
sif{3,1} = 'Scenario name';
sif{4,1} = 'CIMPL-filename';
sif{5,1} = 'Number of reads';
sif{6,1} = 'Number of tumors';
sif{7,1} = 'Linkage type';
sif{8,1} = 'Linkage distance';
sif{9,1} = 'Contigs weighing';
sif{10,1} = 'divisor';
sif{11,1} = 'Iterations';
sif{12,1} = 'Kernel Widths';
sif{13,1} = 'Advanced settings';
sif{14,1} = 'Comments';
tmp = datestr(now);
datestring = tmp(1:findstr(tmp,' ')-1);
timestring = tmp(findstr(tmp,' ')+1:end);
sif{1,2} = datestring;
sif{2,2} = timestring;
sif{3,2} = scenario_name;
sif{4,2} = ' ';
sif{5,2} = num2str(length(data_struct.chr));
try
    sif{6,2} = num2str(length(unique(data_struct.tissueID)));
catch
    sif{6,2} = ' ';
end
sif{7,2} = ' ';
sif{8,2} = ' ';
sif{9,2} = ' ';
sif{10,2} = num2str(divisor);
sif{11,2} = num2str(iterations);
sif{12,2} = num2str(kernel_widths);
sif{13,2} = num2str([sample_window sample_period]);
sif{14,2} = ' ';

fid = fopen([scenario_name '.sif'],'w+');
for strcnt = 1:size(sif,1)
    fprintf(fid,'%s\n',[sif{strcnt,1} ';' sif{strcnt,2}]);
end
fclose(fid);
