%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author            : Jeroen de Ridder
% Copyright         : J. de Ridder, J.deRidder@tudelft.nl
%                     Delft University of Technology
%                     P.O. Box 5031, 2600 GA Delft,
%          
%                     The Netherlands Cancer Institute
%                     Plesmanlaan 121, 1066 CX, Amsterdam,
%                     The Netherlands
% 
% Reference paper   : de Ridder, J, Kool, J, Uren, A,
%                     Wessels, L, Reinders, M.
%                     Detecting statistically significant 
%                     common insertion sites in retroviral
%                     insertional mutagenesis screens.
%                     PLoS Comput Biol, 2006, 2, e166
%
%  function : CIS_analyze(data,kernel_width,null_distr,...
%             sample_period,sample_window)
%  Purpose  : Determine pvalues for all the sample points
%  Inputs   : Insertions (contigs build)
%             sample_window
%             sample_period
%             kernel width
%             Null distribution
%
%  Outputs  : pvalues, densities, sample points and
%             peak positions at specified kernel widths
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [pvals, densities, sample_points, peak_pos, peak_height] = ...
    CIS_analyze(data,kernel_width,null_distr,...
    sample_period,sample_window)

N = length(data);
data = sort(data);
for jj = 1:length(kernel_width)
    h = kernel_width(jj); %Select kernel_width under investigation
    sample_w = sample_window*h; %Normalize the sample window
    sample_p = sample_period*h; %Normalize the sample period
    xtemplate = -sample_w:sample_p:+sample_w;
    xtemplate = repmat(xtemplate,N,1);
    %Determine the sample points
    x = [];
    x = xtemplate+repmat(data,1,size(xtemplate,2));
    x = floor(x(:)/sample_p)*sample_p;
    x = x(x>0);
    x = [x(:); data(:)];
    x = unique(x);
    sample_points_global = x;
    %Do the kernel convolution
    densities_global = kernelconv_fast(data,x,h);
    %Determine the local maxima
    [max_pos, max_val] = get_local_max(densities_global);

    %Recompute densities near the peaks
    sample_w = sample_p;
    sample_p = ceil(0.01*sample_w);
    xtemplate = -sample_w:sample_p:sample_w;
    xtemplate = repmat(xtemplate,length(max_pos),1);
    %Determine the sample points
    x = [];
    x = xtemplate+repmat(sample_points_global(max_pos),1,size(xtemplate,2));
    x = x(x>0);
    x = unique(x);
    [sample_points_fine,I] = sort([sample_points_global; x]);
    %Do the kernel convolution
    tmp = [densities_global; kernelconv_fast(data,x,h)];
    [max_pos, max_val] = get_local_max(tmp(I));
    peak_pos{jj} = sample_points_fine(max_pos);
    
    xCDF = null_distr.xCDF{jj};
    yCDF = null_distr.yCDF{jj};

    warning off
    for j = 1:length(max_val)
        I = find(max_val(j)<xCDF);
        if numel(I)==0
            warning('Threshold value below emperical bounds');
            pvals{jj}(j) = 0;
            peak_height{jj}(j) = max_val(j);
        elseif max_val(j)>(xCDF(I(1))+xCDF(I(1)-1))/2
            pvals{jj}(j) = 1-yCDF(I(1));
            peak_height{jj}(j) = max_val(j);
        else
            pvals{jj}(j) = 1-yCDF(I(1)-1);
            peak_height{jj}(j) = max_val(j);
        end
    end
    [sample_points{jj},I] = sort([sample_points_global; peak_pos{jj}]);
    tmp = [densities_global; peak_height{jj}'];
    densities{jj} = tmp(I); 
end


