function [rhohat, pi_ls, pi_rs, pi_ts, pi_adv] = FisherExact(T);
% [rhohat, pi_ls, pi_rs, pi_ts, pi_adv] = FisherExact(T):
% For a 2-by-2 table T, this program computes the
% estimated odds ratio
%    rhohat  =  (T(1,1) + .5) * (T(2,2) + .5)
%               / (T(1,2) + .5) / (T(2,1) + .5) .
% In addition it computes exact one- and two-sided
% p-values for the null hypothesis that the underlying
% odds ratio rho equals zero, based on the test
% statistic T(1,1).
% As for the two-sided p-values there are two versions:
% 1. Dual to confidence interval:
%       pi_ts  :=  min( 2 * min(pi_ls, pi_rs), 1) .
%    This p-value is less than alpha if, and only if, the
%    (1 - alpha)-confidence interval for rho, as produced
%    for instance by OddsRatioCBs.m, does not contain 1.
% 2. Advanced:
%    Let f(t) be the probability weight of the number t
%    under the hypergeometric distribution Hyp(n, n1, n2),
%    where n1 = T(1,1) + T(2,1), n2 = T(1,1) + T(1,2).
%    Then
%       pi_adv  :=  sum of all weights f(t) which are
%                   not greater than f(T(1,1)).
%    Because of potential numerical inaccuracies the
%    threshold f(T(1,1)) is multiplied with 1.0001. 
% 
% Lutz Duembgen, August 6, 2002

n = sum(sum(T));
n1 = T(1,1) + T(1,2);
n2 = T(1,1) + T(2,1);

rhohat = (T(1,1) + 0.5)*(T(2,2) + 0.5) / (T(1,2) + 0.5) / (T(2,1) + 0.5);

f = LocalPDF(n, n1, n2);
F = cumsum(f);

pi_ls = F(T(1,1) + 1);
if T(1,1) > 0
	pi_rs = 1 - F(T(1,1));
else
	pi_rs = 1;
end
pi_ts = min(2*min(pi_ls, pi_rs), 1);

pi_adv = sum(f(find(f <= f(T(1,1) + 1)*1.0001)));

if nargout == 0
	fprintf(1,'Estimated odds ratio: %5.3f\n', rhohat)
	fprintf(1,'Left-sided P-value:   %5.3f\n', pi_ls)
	fprintf(1,'Right-sided P-value:  %5.3f\n', pi_rs)
	fprintf(1,'Two-sided P-value:    %5.3f  (dual to C.I.)\n', pi_ts)
	fprintf(1,'Two-sided P-value:    %5.3f  (advanced)\n', pi_adv)
end
return

%%%%%%%%%%%%%%%%%%%%%%
% Auxiliary program: %
%%%%%%%%%%%%%%%%%%%%%%

function f = LocalPDF(NN,L,n);
% computes the probability weights
%    f(i+1)  =  Hyp(NN,L,n){i}
% for i=0,1,2,...,min(L,n).

xmax = min(L,n);
f = ones(1,xmax+1);
% At first, f(i) contains the probability weight of {i-1} up
% to a norming factor.

% Starting point near the argmax of the weight function:
start = floor(n*L/NN);

% Compute raw probability weights to the right of start:
tmp = [start+1:xmax];
f(tmp+1) = cumprod( ((n+1)./tmp - 1) .* ((L+1-tmp)./(NN-L-n+tmp)) );

% Compute raw probability weights to the left of start:
tmp = [start:-1:1];
f(tmp) = cumprod( ((NN-L-n+tmp)./(L+1-tmp)) ./ ((n+1)./tmp - 1) );

% normalize f:
f = f / sum(f);
return
