%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author            : Jeroen de Ridder
% Copyright         : J. de Ridder, J.deRidder@tudelft.nl
%                     Delft University of Technology
%                     P.O. Box 5031, 2600 GA Delft,
%          
%                     The Netherlands Cancer Institute
%                     Plesmanlaan 121, 1066 CX, Amsterdam,
%                     The Netherlands
% 
% Reference paper   : de Ridder, J, Kool, J, Uren, A,
%                     Wessels, L, Reinders, M.
%                     Detecting statistically significant 
%                     common insertion sites in retroviral
%                     insertional mutagenesis screens.
%                     PLoS Comput Biol, 2006, 2, e166
%
%  function : nullperm(data,genomelength,
%                   kernelwidths,noperms,divisor,
%                   sampleperiod,samplewindow)
%  Purpose  : Permute the data to acquire null-distribution
%  Inputs   : processes contigs
%             genome length
%             array of kernel widths
%             number of permutations
%             divisor --> reduce computational load
%             sample period multiplication factor
%             sample window multiplication factord
%
%  Outputs  : emperical CDF per kernel width
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function nulldistr = nullperm(data,genomelength,kernel_widths,noperms,perm_split,divisor,...
    sample_period,sample_window)

if ispc
    DIR_SYM = '\';
else
    DIR_SYM = '/';
end


G = round(genomelength/divisor); %determine artificial genome length
if length(data)>1
    N = round(length(data)/divisor); %determine artificial number of inserts
else
    N = round(data/divisor);
end

%Make temporary dir
mkdir('perm_dump');

for jj = 1:length(kernel_widths)
    fprintf('\n Permuting... current kernel width: %d\n',kernel_widths(jj));
    h = kernel_widths(jj); %Select a kernel width
    sample_w = sample_window*h/2;
    sample_p = sample_period*h;
    %disp('kernelwidth')
    xtemplate = -sample_w:sample_p:+sample_w;
    xtemplate = repmat(xtemplate,N,1);
    local_maximums = -ones(perm_split,N);
    for filestep = 1:noperms/perm_split
        fprintf('.');
        for j = 1:perm_split
            permdata = sort(round(G*rand(N,1))); %Generate a permutation
            %Compute sample points
            x = [];
            x = xtemplate+repmat(permdata,1,size(xtemplate,2));
            x = floor(x(:)/sample_p)*sample_p;
            x = x(x>0);
            x = [x(:); permdata(:)];
            x = unique(x);
            %Do the kernel convolution
            KC = kernelconv_fast(permdata,x,h);
            [max_pos, max_val] = get_local_max(KC); %Determine the peaks
            local_maximums(j,1:length(max_val)) = max_val;
        end
        %Convert to pseudo CDF (not normalized CDF)
        loc_max = local_maximums(:);
        loc_max = loc_max(loc_max~=-1);
        eval(['save perm_dump' DIR_SYM 'kw' num2str(jj) '_file' num2str(filestep) ' loc_max']);
    end
    xCDF{jj} = [];
    yCDF{jj} = [];
    for filestep = 1:noperms/perm_split
        %Load the saved part of the permutation
        eval(['load perm_dump' DIR_SYM 'kw' num2str(jj) '_file' num2str(filestep)]);
        %(re)build the bins (unique x vector)
        orig_N = length(xCDF{jj});
        if orig_N==0
            xCDF{jj} = unique(loc_max);
            yCDF{jj} = histc(loc_max,xCDF{jj});
        else
            %Recompute the unique bins
            [xCDF{jj},dummy,I] = unique([xCDF{jj}; loc_max]);
            %Redistibute what we already knew
            yCDFtmp = zeros(length(xCDF{jj}),1);
            yCDFtmp(I(1:orig_N)) = yCDF{jj};
            %Compute the new observations and add to the old
            yCDF{jj} = histc(loc_max,xCDF{jj}) + yCDFtmp(:);
        end
    end
    yCDF{jj} = cumsum(yCDF{jj});
    yCDF{jj} = yCDF{jj}/yCDF{jj}(end);
end
nulldistr.xCDF = xCDF;
nulldistr.yCDF = yCDF;

%Clean up the files
delete(['perm_dump' DIR_SYM '*.mat']);
rmdir('perm_dump');