%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author            : Jeroen de Ridder
% Copyright         : J. de Ridder, J.deRidder@tudelft.nl
%                     Delft University of Technology
%                     P.O. Box 5031, 2600 GA Delft,
%          
%                     The Netherlands Cancer Institute
%                     Plesmanlaan 121, 1066 CX, Amsterdam,
%                     The Netherlands
% 
% Reference paper   : de Ridder, J, Kool, J, Uren, A,
%                     Wessels, L, Reinders, M.
%                     Detecting statistically significant 
%                     common insertion sites in retroviral
%                     insertional mutagenesis screens.
%                     PLoS Comput Biol, 2006, 2, e166
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [output] = scale_proc(Tlevel_all,densities,peak_pos,kernel_widths,...
    sample_points,alpha_threshold,chrom_l,cross_scale)
%Compute the Common Insertion sites
CIS_scale   = [];
CIS_all     = [];
dC = [];
for k_ind = 1:length(peak_pos)
    T_level         = Tlevel_all(k_ind);
    
    [max_pos_I, max_val]    = get_local_max(densities{k_ind});
    [min_pos_I, min_val]    = get_local_min(densities{k_ind});

    max_pos                 = sample_points{k_ind}(max_pos_I(find(max_val>T_level)));
    max_val                 = densities{k_ind}(max_pos_I(find(max_val>T_level)));
    min_pos                 = sample_points{k_ind}(min_pos_I(find(min_val>T_level)));
    min_val                 = densities{k_ind}(min_pos_I(find(min_val>T_level)));
    [rise_pos, descent_pos] = get_intersect(sample_points{k_ind},densities{k_ind},T_level);
    %Build a quadstatevector
    [positions, I]          = sort([max_pos(:); min_pos(:); rise_pos(:); descent_pos(:)]);
    qsv                     = [2*ones(size(max_pos(:))); 8*ones(size(min_pos(:)));...
        ones(size(rise_pos(:))); 4*ones(size(descent_pos(:)))];
    qsv                     = qsv(I);
    %Detect local maxima up to 3 local maxima above threshold
    cnt     = 1;
    I_start = [];
    I_CIS   = [];
    I_end   = [];
    i       = 1;
    while i<=length(qsv)-2
        if qsv(i)==1 & qsv(i+1)==2
            I_start(cnt)    = i;
            I_CIS(cnt)      = i+1;
            end_char        = qsv(i+2);
            while end_char~=4
                I_end(cnt)  = i+2;
                i           = i+2;
                cnt         = cnt+1; %Start a NEW CIS
                I_start(cnt)= i;
                I_CIS(cnt)  = i+1;
                end_char    = qsv(i+2);
            end
            I_end(cnt)      = i+2; %Close CIS
            cnt             = cnt+1; %Start a NEW CIS
        end
        i                   = i+1;
    end
    CIS_all{k_ind}          = positions(I_CIS);
    dC = [dC(:); kernel_widths(k_ind)*ones(length(CIS_all{k_ind}),1)];

    CIS_scale      = [CIS_scale; ...
        positions(I_start) positions(I_end)];
end

dC = [dC dC]';
cutoff = max(dC(1,:));
link = 'single';

x1 = mean(CIS_scale');
x_stack = [];
y_stack = [];
clustlabel = [];
for i = 1:21
    I = find(x1>chrom_l(i)&x1<chrom_l(i+1));
    if ~isempty(I)
        x_tmp = x1(I);
        y_tmp = dC(:,I);
        w_tmp = CIS_scale(I,:)';
        distdata = pdist([x_tmp(:) zeros(length(x_tmp))]); %Interdata distance
        if isempty(distdata)
            C = 1;
        elseif size(distdata,2)==1
            if distdata<cutoff
                C = [1 1];
            else
                C = [1 2];
            end
        else
            dend = linkage(distdata,link); %Dendrogram with MATLAB build-in
            C = cluster(dend,'cutoff',cutoff,'criterion','distance'); %Cut at distance
        end
        if isempty(max(clustlabel)), offset = 0; else offset = max(clustlabel); end
        clustlabel = [clustlabel(:); C(:)+offset];
        x_stack = [x_stack [w_tmp(1,:); w_tmp(2,:)]];
        y_stack = [y_stack y_tmp];
    end
end
if cross_scale
    uclust = unique(clustlabel);
    for i = 1:length(uclust)
        I = find(clustlabel==uclust(i));
        cross_scale(i) = mean(mean(x_stack(:,I),2),1);
    end
    output = sort(cross_scale );
else
    scale_struct.y              = y_stack;
    scale_struct.x              = x_stack;
    scale_struct.ylim           = [min(kernel_widths)/2 max(kernel_widths)*10];
    scale_struct.clustlabel     = clustlabel;
    output = scale_struct;
end