%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author            : Jeroen de Ridder
% Copyright         : J. de Ridder, J.deRidder@tudelft.nl
%                     Delft University of Technology
%                     P.O. Box 5031, 2600 GA Delft,
%          
%                     The Netherlands Cancer Institute
%                     Plesmanlaan 121, 1066 CX, Amsterdam,
%                     The Netherlands
% 
% Reference paper   : de Ridder, J, Kool, J, Uren, A,
%                     Wessels, L, Reinders, M.
%                     Detecting statistically significant 
%                     common insertion sites in retroviral
%                     insertional mutagenesis screens.
%                     PLoS Comput Biol, 2006, 2, e166
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [track_struct, dens_struct, scale_struct] = track_processing(handles)
%Set available settings (these include some options that are not currently used):
possible_features = {'CIS','cr_CIS','u_CIS','insertions_pos','insertions_neg','contigs_pos','contigs_neg',...
     'comp_CIS','comp_cr_CIS','comp_u_CIS','comp_insertions_pos','comp_insertions_neg','comp_contigs',...
     'genes','ext'};

%Unpack the MAIN data from handles
track_settings  = handles.track_settings;
kernel_index    = handles.kernel_index;

insertions      = handles.insertions;
orientation     = handles.orientation;
contig_depth    = handles.contig_depth;

Tlevel          = handles.Tlevel;
alpha           = handles.alpha;
cum_chrom_l         = handles.cum_chrom_l;

densities       = handles.densities;
sample_points   = handles.sample_points;
peak_pos        = handles.peak_pos;
peak_height     = handles.peak_height;

kernel_widths   = handles.kernel_widths;
top_gene        = handles.top;
bot_gene        = handles.bot;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DENSITY TRACK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dens_struct     = struct();

dens_struct.y    = densities{kernel_index};
dens_struct.x    = sample_points{kernel_index};
dens_struct.T    = [Tlevel(kernel_index) Tlevel(kernel_index)];
dens_struct.T_l  = '-';
dens_struct.color= track_settings{10}{2};
dens_struct.label= track_settings{10}{3};
dens_struct.type = 'insertion';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SCALE SPACE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale_struct    = scale_proc(Tlevel,densities,peak_pos,kernel_widths,...
    sample_points,alpha,cum_chrom_l,0);
scale_struct.label          = track_settings{11}{3};
scale_struct.color          = track_settings{11}{2};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INSERTION TRACK
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Define the trackstruct
track_struct = struct('numel',cell(1,5),'orientation',cell(1,5),'x',cell(1,5),'color',cell(1,5),...
    'style',cell(1,5),'feature',cell(1,5),'enable',cell(1,5),'label',cell(1,5),'always_show',cell(1,5));

%Find the enabled tracks
for tracknumber = 1:5
    if ~track_settings{tracknumber}{6}(1)
        continue %Skip this, for disabled tracks
    end
    feature_list = possible_features(logical([track_settings{tracknumber}{1} ...
        track_settings{tracknumber}{2}...
        track_settings{tracknumber}{3}]));
    track_struct(tracknumber).numel = length(feature_list);
    track_struct(tracknumber).enable = 1;
    track_struct(tracknumber).always_show = track_settings{tracknumber}{6}(2);

    for j = 1:length(feature_list) %loop over the features
        switch feature_list{j}
            case 'CIS'
                I = peak_height{kernel_index}>Tlevel(kernel_index);
                track_struct(tracknumber).x{j}        = peak_pos{kernel_index}(I);
                track_struct(tracknumber).style       = 'insertion';
                track_struct(tracknumber).feature{j}  = 'CIS';
            case 'cr_CIS'
                cross_scale                 = scale_proc(Tlevel,densities,peak_pos,...
                    kernel_widths,sample_points,alpha_threshold,cum_chrom_l,1);
                track_struct(tracknumber).x{j}        = cross_scale(:);
                track_struct(tracknumber).style       = 'insertion';
                track_struct(tracknumber).feature{j}  = 'cr_CIS';
            case 'u_CIS'
                I = find(pvals{kernel_index}<(alpha_threshold));
                track_struct(tracknumber).x{j}        = peak_pos{kernel_index}(I);
                track_struct(tracknumber).style       = 'insertion';
                track_struct(tracknumber).feature{j}  = 'u_CIS';
            case 'insertions_pos'
                track_struct(tracknumber).x{j}        = insertions(orientation==1&contig_depth==1);
                track_struct(tracknumber).style       = 'insertion';
                track_struct(tracknumber).feature{j}  = 'insertions_pos';
            case 'insertions_neg'
                track_struct(tracknumber).x{j}        = insertions(orientation==-1&contig_depth==1);
                track_struct(tracknumber).style       = 'insertion';
                track_struct(tracknumber).feature{j}  = 'insertions_neg';
            case 'contigs_pos'
                track_struct(tracknumber).x{j}        = insertions(orientation==1&contig_depth>1);
                track_struct(tracknumber).style       = 'insertion';
                track_struct(tracknumber).feature{j}  = 'contigs_pos';
            case 'contigs_neg'
                track_struct(tracknumber).x{j}        = insertions(orientation==-1&contig_depth>1);
                track_struct(tracknumber).style       = 'insertion';
                track_struct(tracknumber).feature{j}  = 'contigs_neg';
            case 'genes'
                %{1} --> [top_5prime top_3prime]
                %{2} --> [bot_3prime bot_5prime]
                track_struct(tracknumber).x{j}{1}     = [top_gene.prime5(:) top_gene.prime3(:)];
                track_struct(tracknumber).x{j}{2}     = [bot_gene.prime3(:) bot_gene.prime5(:)];
                track_struct(tracknumber).style       = 'gene';
                track_struct(tracknumber).feature{j}  = 'genes';
            otherwise
                error('unknown feature style, or feature style not in use');
        end
        track_struct(tracknumber).color{j} = track_settings{tracknumber}{4};
        track_struct(tracknumber).label    = track_settings{tracknumber}{5};
    end
end